/* kbbios.c -- BIOS keyboard access
 * Copyright (C) 1995, 1996 Markus F.X.J. Oberhumer
 * For conditions of distribution and use, see copyright notice in kb.h 
 */

#if defined(__EMX__)
#  include <sys/emx.h>		/* must be first include file */
#  include <stdlib.h>
#endif
#include <kb.h>
#include "_kb.h"


/***********************************************************************
//
************************************************************************/

#if defined(__EMX__) || defined(__GO32__)

static __inline__ unsigned _my_bios_keybrd(unsigned cmd)
{
	union REGS regs;

#if defined(__EMX__)
	if ((_emx_env & 0x0200) != 0)
	{
		/* err_internal("emx: not in DOS_MODE"); */
		return 0;
	}
	if ((_emx_env & 0x0800) == 0)
	{
		/* err_internal("emx: -ac not enabled for _int86()"); */
		return 0;
	}
#endif /* __EMX__ */

	regs.h.al = 0;
	regs.h.ah = cmd;
	KB_INT86(0x16,&regs,&regs);

	if (cmd == KB_KEYBRD_READY || cmd == KB_NKEYBRD_READY)
	{
		if (regs.x.flags & 0x40)	/* zero flag set -> no key is waiting */
			return 0;
		else if (regs.x.ax == 0)
			return 0x0000ffff;		/* Control-Break */
		else
			return regs.x.ax;
	}
	else
		return regs.x.ax;
}

#define _KB_BIOSKEY(x)	_my_bios_keybrd(x)

#endif /* __EMX__ || __GO32__ */


#if defined(MSDOS) || defined(__MSDOS__)
#  if defined(__BORLANDC__) || defined(__TURBOC__) || defined(__GO32__)
#    ifndef _KB_BIOSKEY
#      define _KB_BIOSKEY(x)	bioskey(x)
#    endif
#  endif
#  if defined(_MSC_VER) || defined(__WATCOMC__)
#    ifndef _KB_BIOSKEY
#      define _KB_BIOSKEY(x)	_bios_keybrd(x)
#    endif
#  endif
#endif



/***********************************************************************
//
************************************************************************/

unsigned kb_bioskey(unsigned cmd)
{
#if defined(_KB_BIOSKEY)
	return _KB_BIOSKEY(cmd);
#else
	return 0;
#endif
}


int kb_bios_kbhit(void)
{
	if (_kb_mode)
		return kb_kbhit();

#if defined(_KB_BIOSKEY)
	return (_KB_BIOSKEY(KB_NKEYBRD_READY) != 0);
#else
	return kbhit();
#endif
}


int kb_bios_getch(void)
{
	if (_kb_mode)
		return kb_getch();

#if defined(_KB_BIOSKEY)
{
	static unsigned last_key = 0;
	unsigned c;

	if (last_key == 0)
	{
		c = _KB_BIOSKEY(KB_NKEYBRD_READ);
		if ((c & 0xff) == 0x00 || (c & 0xff) == 0xe0) 
		{
			last_key = c >> 8;
			c = 0;
		}
		else
			c &= 0xff;
	}
	else
	{
		c = last_key;
		last_key = 0;
	}
	return c;
}
#else
	return getch();
#endif
}


/***********************************************************************
// emx+gcc lacks a real kbhit() and getch()
************************************************************************/

#if defined(__EMX__)

static int _last_key = -1;


int kbhit(void)
{
#if 1
	/* use BIOS if possible, cause it's MUCH faster */
	if ((_emx_env & (0x0200 | 0x0800)) == 0x0800)
		return kb_bios_kbhit();
#endif
	if (_last_key == -1)
		_last_key = _read_kbd(0, 0, 0);
	return (_last_key != -1);
}


int getch(void)
{
	int c;

	if (_last_key == -1)
		c = _read_kbd(0, 1, 0);
	else
	{
		c = _last_key;
		_last_key = -1;
	}
	return c;
}

#endif /* __EMX__ */



/*
vi:ts=4
*/
